using System;
using System.Collections;
using System.Drawing;
using System.Drawing.Drawing2D;

using pdftron;
using pdftron.PDF;
using pdftron.Common;


namespace PDFViewCS
{
	/// <summary>
	/// Summary description for FreeHandAnnot.
	/// </summary>
	// A FreeHandAnnot is a list of FreeHand polygon strokes.
	public class FreeHandAnnot
	{		
		private ArrayList _paths = new ArrayList();

		public void AddPath(GraphicsPath path)
		{
			_paths.Add(path);
		}

		public void DrawAnnots(Graphics g, Pen pen)
		{
			for(int i=0; i<_paths.Count; ++i) // Draw all FreeHand strokes
			{						
				g.DrawPath(pen, (GraphicsPath) _paths[i]);
			}
		}

		private ElementBuilder _builder = new ElementBuilder();
		private ElementWriter _writer = new ElementWriter();

		public void DrawAnnots(Page page, Pen pen)
		{
			_writer.Begin(page);  // Begin writing to PDF page
			_builder.Reset(); 	  // Reset GState to default graphics state

			Matrix2D inv_mtx = page.GetDefaultMatrix();
			double x, y;

			for(int i=0; i<_paths.Count; ++i) // Draw all FreeHand strokes
			{
				GraphicsPath p = (GraphicsPath) _paths[i];

				GraphicsPathIterator pitr = new GraphicsPathIterator(p);
				if (pitr.Count>0) 
				{
					bool IsClosed;
					int numSubpaths = pitr.NextSubpath(p, out IsClosed);

					_builder.PathBegin();

					byte subPathPointType;
					int pointTypeStartIndex,  pointTypeEndIndex, numPointsFound;

					while ((numPointsFound = pitr.NextPathType(out subPathPointType, out pointTypeStartIndex, out pointTypeEndIndex)) != 0) 
					{
						if (subPathPointType == (byte) PathPointType.Line) 
						{
							x = p.PathPoints[pointTypeStartIndex].Y; y = p.PathPoints[pointTypeStartIndex].X;
							inv_mtx.Mult(ref x, ref y);
							_builder.MoveTo(x, y);
							for (int j=pointTypeStartIndex+1; j<=pointTypeEndIndex; ++j)
							{
								x = p.PathPoints[j].Y; y = p.PathPoints[j].X;
								inv_mtx.Mult(ref x, ref y);
								_builder.LineTo(x, y);
							}
						}
					}

					Element element = _builder.PathEnd();	// path geometry is completed
					element.SetPathFill(false);		// this path is should not filled
					element.SetPathStroke(true);

					// Set the path color space and color
					GState gstate = element.GetGState();
					gstate.SetStrokeColorSpace(ColorSpace.CreateDeviceRGB()); 
					Color pc = pen.Color;
					gstate.SetStrokeColor(new ColorPt(pc.R/255.0, pc.G/255.0, pc.B/255.0));
					gstate.SetLineWidth(1);
					_writer.WriteElement(element);
				}
			}

			_writer.End();  // save changes to the page
		}
	}
}
