Imports System
Imports System.Drawing
Imports System.Drawing.Drawing2D
Imports System.Collections
Imports System.ComponentModel
Imports System.Windows.Forms
Imports PDFTRON
Imports PDFTRON.PDF
Imports PDFTRON.Common
Imports PDFTRON.SDF
Imports PDFTRON.PDF.Annots
'<summary>
'MyPDFView is a sample class derived from pdftron.PDF.PDFViewCtrl 
'that illustrates how to implement various custom tools and custom 
'drawing on top of PDFViewCtrl. If you only need to view PDF documents 
'you can completely comment-out this class and instantiate 
'pdftron.PDF.PDFViewCtrl directly.
'MyPDFView shows how to implement various custom PDF tools such as a 
'tool to create new link annotations, a rectangular zoom, a freehand draw,
'link navigation, etc).
'</summary>
Public Class MyPDFView
    Inherits PDFViewCtrl


    Public Enum CustomToolMode
        e_none
        e_custom_draw        ' Custom Markup tool
        e_custom_link_create ' Custom Link-create tool
        e_custom_zoomin      ' Custom Rectangular zoom-in tool
        e_custom_zoomout     ' Custom Rectangular zoom-out tool
        e_custom_link_action
        e_custom_mouse_hover
        ' <-- Add new tool modes 
    End Enum

    Public Shared _tool_mode As CustomToolMode = CustomToolMode.e_none
    Public Shared _base_tool_mode As ToolMode = ToolMode.e_pan
    Private _parent As PDFViewForm

    Public Sub New(ByRef parent As PDFViewForm)
        MyBase.New()

        _parent = parent
        AddHandler _popup.Popup, New EventHandler(AddressOf PopupEventHandler)
        Me.ContextMenu = _popup

        ShowNavPanel(True)
        'SetProgressiveRendering(false);
        'SetDrawAnnotations(false);
    End Sub

    Protected Overloads Overrides Sub OnPaint(ByVal e As PaintEventArgs)

        If (_tool_mode = CustomToolMode.e_custom_link_create OrElse _tool_mode = CustomToolMode.e_custom_zoomin OrElse _tool_mode = CustomToolMode.e_custom_zoomout) AndAlso Capture AndAlso _cur_page > 0 Then
            ' Draw Link rectangle annotations on top of the PDF page.
            Dim top As Single
            Dim bottom As Single
            Dim left As Single
            Dim right As Single
            If _start_pt.Y < _end_pt.Y Then
                top = _start_pt.Y
                bottom = _end_pt.Y
            Else
                bottom = _start_pt.Y
                top = _end_pt.Y
            End If
            If _start_pt.X < _end_pt.X Then
                left = _start_pt.X
                right = _end_pt.X
            Else
                right = _start_pt.X
                left = _end_pt.X
            End If
            e.Graphics.DrawRectangle(Pens.Red, left, top, right - left, bottom - top)
        End If
        If Not (_freehand_markup Is Nothing) Then ' Draw FreeHand annotations on top.
            Dim pmode As PagePresentationMode = GetPagePresentationMode()
            Dim curpage As Integer = GetCurrentPage()
            Dim g As Graphics = e.Graphics
            For Each itr As DictionaryEntry In _freehand_markup
                Dim annot_page_num As Integer = CType(itr.Key, Integer)
                If Not (pmode = PDFViewCtrl.PagePresentationMode.e_single_page AndAlso Not (annot_page_num = curpage)) OrElse (pmode = PDFViewCtrl.PagePresentationMode.e_facing AndAlso Not (annot_page_num = curpage) AndAlso Not (annot_page_num = curpage + 1)) Then
                    Dim gs As GraphicsState = g.Save
                    Dim page2scr_mtx As Matrix = GetDeviceTransform(annot_page_num)
                    g.MultiplyTransform(page2scr_mtx)
                    Dim annot As FreeHandAnnot = CType(itr.Value, FreeHandAnnot)
                    annot.DrawAnnots(g, Pens.Red)
                    g.Restore(gs)
                End If
            Next
        End If
    End Sub

    Protected Overloads Overrides Sub OnPaintBackground(ByVal pevent As PaintEventArgs)
    End Sub
    Protected _start_pt As PointF
    Protected _end_pt As PointF
    Protected _cur_page As Integer = 0

    ' A dictionary mapping page numbers to FreeHandAnnot-s (used to implement freehand markup tool)
    Public _freehand_markup As Hashtable = Nothing
    Protected _curstroke As GraphicsPath = Nothing ' Used to track the current stroke.

    Protected Overloads Overrides Sub OnMouseDown(ByVal e As MouseEventArgs)

        MyBase.OnMouseDown(e)  ' Call the base class

        If (_tool_mode = CustomToolMode.e_custom_link_create OrElse _tool_mode = CustomToolMode.e_custom_zoomin OrElse _tool_mode = CustomToolMode.e_custom_zoomout) Then
            _cur_page = GetPageNumberFromScreenPt(e.X, e.Y)
            If _cur_page < 1 Then
                _cur_page = GetCurrentPage()
            End If

            ' Record the position for the rectangle origin.
            _start_pt.X = e.X
            _start_pt.Y = e.Y
            Capture = True
        ElseIf _tool_mode = CustomToolMode.e_custom_draw Then
            ' Sample code illustrating how to draw new content on top 
            ' of PDF page...
            _cur_page = GetPageNumberFromScreenPt(e.X, e.Y)
            If _cur_page < 1 Then
                _cur_page = GetCurrentPage()
            End If
            If _freehand_markup Is Nothing Then
                _freehand_markup = New Hashtable
            End If
            Dim annot As FreeHandAnnot
            If Not _freehand_markup.ContainsKey(_cur_page) Then
                annot = New FreeHandAnnot
                _freehand_markup.Add(_cur_page, annot)
            Else
                annot = CType(_freehand_markup(_cur_page), FreeHandAnnot)
            End If
            _curstroke = New GraphicsPath ' Start a new stroke.
            annot.AddPath(_curstroke)

            ' Record the current position.
            Dim x As Double = e.X
            Dim y As Double = e.Y
            Me.ConvScreenPtToPagePt(x, y, _cur_page)
            _start_pt.X = CType(x, Single)
            _start_pt.Y = CType(y, Single)
            Capture = True
        ElseIf (_tool_mode = CustomToolMode.e_custom_link_action) Then
            ' Sample code illustrating how to implement link navigation.
            If IsFinishedRendering() Then
                Dim page_num As Integer = GetPageNumberFromScreenPt(e.X, e.Y)
                If page_num < 1 Then
                    Return
                End If

                ' Find the click point in page coordinate system...
                Dim x As Double = e.X
                Dim y As Double = e.Y
                ConvScreenPtToPagePt(x, y, page_num)
                Dim page As Page = GetDoc.GetPage(page_num)
                Dim annot_num As Integer = page.GetNumAnnots
                Dim i As Integer = 0
                While i < annot_num
                    Dim annot As Annot = page.GetAnnot(i)
                    ' Process only link annotations...
                    If annot.IsValid = False OrElse Not (annot.GetType = annot.Type.e_Link) Then
                        'continue at the next iteration of the loop
                        GoTo Continue1
                    End If
                    Dim box As Rect = annot.GetRect
                    If box.Contains(x, y) Then
                        ' Execute the action - jump to the destination page
                        Dim lk As Link = New pdftron.PDF.Annots.Link(annot.GetSDFObj())
                        Dim action As Action = lk.GetAction()
                        If action.IsValid = False Then
                            'continue at the next iteration of the loop
                            GoTo Continue1
                        End If
                        If action.GetType = action.Type.e_GoTo Then
                            page_num = action.GetDest.GetPage.GetIndex
                            SetCurrentPage(page_num)
                            Return
                        ElseIf action.GetType = action.Type.e_URI Then
                            ' open the browser and jump to the given hyperlink...
                            Dim target As String

                            target = action.GetSDFObj().Get("URI").Value().GetAsPDFText()
                            Try
                                System.Diagnostics.Process.Start(target)
                            Catch noBrowser As System.ComponentModel.Win32Exception
                                If noBrowser.ErrorCode = -2147467259 Then
                                    MessageBox.Show(noBrowser.Message)
                                End If
                            Catch other As System.Exception
                                MessageBox.Show(other.Message)
                            End Try

                            'Break out of the loop
                            GoTo Break
                        ElseIf action.GetType = action.Type.e_GoToR Then
                            ' if the remote document is PDF, possibly open the
                            ' document as a new PDFViewForm...
                        End If
                    End If

Continue1:
                    i = i + 1                'increment i
                End While
Break:
            End If
        End If
    End Sub

    Protected Overloads Overrides Sub OnMouseMove(ByVal e As MouseEventArgs)

        MyBase.OnMouseMove(e)  ' Call the base class

        If (_tool_mode = CustomToolMode.e_custom_link_create OrElse _tool_mode = CustomToolMode.e_custom_zoomin OrElse _tool_mode = CustomToolMode.e_custom_zoomout) AndAlso Capture AndAlso _cur_page > 0 Then
            _end_pt.X = e.X
            _end_pt.Y = e.Y
            Invalidate()
        Else
            If _tool_mode = CustomToolMode.e_custom_mouse_hover Then
                ' Sample code showing how to change mouse cursor when hovering
                ' over link annotations (in pan mode)...
                If IsFinishedRendering() Then
                    Dim page_num As Integer = GetPageNumberFromScreenPt(e.X, e.Y)
                    If page_num < 1 Then
                        Return
                    End If
                    ' Find the point in PDF page coordinate system...
                    Dim x As Double = e.X
                    Dim y As Double = e.Y
                    ConvScreenPtToPagePt(x, y, page_num)
                    Dim page As Page = GetDoc.GetPage(page_num)
                    Dim annot_num As Integer = page.GetNumAnnots
                    Dim over_link As Boolean = False
                    Dim i As Integer = 0
                    While i < annot_num
                        Dim annot As Annot = page.GetAnnot(i)
                        If annot.IsValid = False OrElse Not (annot.GetType = annot.Type.e_Link) Then
                            'continues on the next iteration of the loop
                            GoTo Continue1
                        End If
                        Dim box As Rect = annot.GetRect
                        If box.Contains(x, y) Then
                            over_link = True
                            'breaks out of the loop
                            GoTo Break
                        End If
Continue1:
                        i = i + 1
                    End While
Break:
                    If over_link Then
                        Cursor = Cursors.Cross
                    Else
                        Cursor = Cursors.Hand
                    End If
                End If
            Else
                If _tool_mode = CustomToolMode.e_custom_draw AndAlso Capture AndAlso Not (_curstroke Is Nothing) Then
                    Dim x As Double = e.X
                    Dim y As Double = e.Y
                    Me.ConvScreenPtToPagePt(x, y, _cur_page)
                    _end_pt.X = CType(x, Single)
                    _end_pt.Y = CType(y, Single)
                    _curstroke.AddLine(_start_pt.Y, _start_pt.X, _end_pt.Y, _end_pt.X)
                    _start_pt.X = _end_pt.X
                    _start_pt.Y = _end_pt.Y
                    Invalidate()
                End If
            End If
        End If
    End Sub

    Protected Overloads Overrides Sub OnMouseUp(ByVal e As MouseEventArgs)

        MyBase.OnMouseUp(e)  ' Call the base class

        If _tool_mode = CustomToolMode.e_custom_link_create AndAlso Capture AndAlso _cur_page > 0 Then
            ' The following sample code illustrates an implementation of link annotation 
            ' tool that can be used to create new links on top of existing PDF documents.
            _end_pt.X = e.X
            _end_pt.Y = e.Y
            Dim dlg As LinkCreate = New LinkCreate
            If dlg.ShowDialog = DialogResult.OK Then
                Dim doc As PDFDoc = GetDoc()
                Dim pg As Page = doc.GetPage(_cur_page)
                If pg Is Nothing Then
                    If dlg.Text = "" Then
                        MessageBox.Show("The link text was not entered.", "PDFViewCtrl", MessageBoxButtons.OK)
                    Else
                        Dim action As Action = Nothing
                        If dlg.goto_webpage.Checked Then
                            If dlg.link.Text.StartsWith("http://") Then
                                ' Create a URI action
                                Dim a As pdftron.SDF.Obj = doc.CreateIndirectDict
                                a.PutName("S", "URI")
                                a.PutString("URI", dlg.link.Text)
                                action = New Action(a)
                            Else
                                MessageBox.Show("The hyperlink must start with 'http://'", "PDFViewCtrl", MessageBoxButtons.OK)
                            End If
                        Else
                            Try
                                Dim page_num As Integer = System.Convert.ToInt32(dlg.link.Text)
                                If page_num < 1 OrElse page_num > doc.GetPageCount Then
                                    MessageBox.Show("Invalid page number.", "PDFViewCtrl", MessageBoxButtons.OK)
                                Else
                                    Dim dest_page As Page = doc.GetPage(page_num)
                                    action = action.CreateGoto(Destination.CreateFitH(dest_page, dest_page.GetPageHeight))
                                End If
                            Catch generatedExceptionVariable0 As Exception
                                MessageBox.Show("Invalid page number.", "PDFViewCtrl", MessageBoxButtons.OK)
                            End Try
                        End If
                        If Not (action Is Nothing) Then
                            ' Create link annotation position rectangle in PDF page coordinate system.
                            Dim screen2page As Matrix = GetDeviceTransform(_cur_page)
                            screen2page.Invert()
                            Dim pt_arr As PointF() = {_start_pt, _end_pt}
                            screen2page.TransformPoints(pt_arr)
                            Dim pos As Rect = New Rect(pt_arr(0).Y, pt_arr(0).X, pt_arr(1).Y, pt_arr(1).X)
                            pos.Normalize()
                            Dim link As Link = link.Create(doc.GetSDFDoc, pos, action)

                            ' Todo? 
                            ' Create link appearances based on user preferences.
                            ' Create other annotation and action types...
                            ' ...

                            pg.AnnotPushBack(link)
                        End If
                    End If
                End If
            End If
            Capture = False
            _cur_page = 0
            Invalidate()
        ElseIf (_tool_mode = CustomToolMode.e_custom_zoomin OrElse _tool_mode = CustomToolMode.e_custom_zoomout) AndAlso Capture AndAlso _cur_page > 0 Then
            ' The following sample code illustrates an implementation of rectangular 
            ' zoom-in and zoom-out tool (similar to the one in Acrobat Reader).

            _end_pt.X = e.X
            _end_pt.Y = e.Y
            Dim zoom_width As Double = _end_pt.X - _start_pt.X
            Dim zoom_height As Double = _end_pt.Y - _start_pt.Y

            If (Math.Abs(zoom_width) > 20 And Math.Abs(zoom_height) > 20) Then
                Dim z1 As Double
                Dim z2 As Double
                If _tool_mode = CustomToolMode.e_custom_zoomin Then
                    z1 = Width / Math.Abs(zoom_width)
                    z2 = Height / Math.Abs(zoom_height)
                Else
                    z1 = Math.Abs(zoom_width) / Width
                    z2 = Math.Abs(zoom_height) / Height
                End If
                Dim z As Double = Math.Min(z1, z2) * GetZoom()
                SetZoom(CType((_start_pt.X + zoom_width / 2), Integer), CType((_start_pt.Y + zoom_height / 2), Integer), z)
            End If

            Capture = False
            _cur_page = 0
            Invalidate()

        ElseIf _tool_mode = CustomToolMode.e_custom_draw AndAlso Capture Then
            ' Freehand tool
            Dim x As Double = e.X
            Dim y As Double = e.Y
            Me.ConvScreenPtToPagePt(x, y, _cur_page)
            _curstroke.AddLine(_start_pt.Y, _start_pt.X, CType(y, Single), CType(x, Single))
            Invalidate()
            _curstroke = Nothing
            Capture = False
            Invalidate()
        End If
    End Sub

    '' Override built-in double click event?
    'Protected Overloads Overrides Sub OnDoubleClick(ByVal e As EventArgs)
    '    MessageBox.Show("Custom OnDoubleClick handler", "Test", MessageBoxButtons.OK)
    'End Sub

    '' Override built-in mouse wheel event handling?
    'Protected Overloads Overrides Sub OnMouseWheel(ByVal e As MouseEventArgs)
    '    If e.Delta > 0 Then
    '        GotoPreviousPage()
    '    ElseIf e.Delta < 0 Then
    '        GotoNextPage()
    '    End If
    'End Sub

    Private _popup As ContextMenu = New ContextMenu

    Public Sub PopupEventHandler(ByVal sender As System.Object, ByVal e As System.EventArgs)

        _popup.MenuItems.Clear()    ' Clear all previously added MenuItems.

        If HasSelection() Then ' Add MenuItems to display if there is a text selection.
            Dim menuItem1 As MenuItem = New MenuItem("Cop&y")
            AddHandler menuItem1.Click, New EventHandler(AddressOf OnTextCopy)
            _popup.MenuItems.Add(menuItem1)
        End If

        If (_tool_mode = CustomToolMode.e_custom_mouse_hover) Then
            Dim menuItem2 As MenuItem = New MenuItem("Se&lect All")
            AddHandler menuItem2.Click, New EventHandler(AddressOf OnSelectAll)
            _popup.MenuItems.Add(menuItem2)

            Dim menuItem3 As MenuItem = New MenuItem("D&eselect All")
            AddHandler menuItem3.Click, New EventHandler(AddressOf OnDeselectAll)
            _popup.MenuItems.Add(menuItem3)
        End If

        Dim menuItem4 As MenuItem = New MenuItem("Print")
        AddHandler menuItem4.Click, New EventHandler(AddressOf OnPrint)
        _popup.MenuItems.Add(menuItem4)
    End Sub

    Public Sub OnTextCopy(ByVal sender As System.Object, ByVal e As System.EventArgs)
        Dim sel As Selection = GetSelection()
        Clipboard.SetDataObject(sel.GetAsUnicode())
    End Sub

    Private Sub OnSelectAll(ByVal sender As System.Object, ByVal e As System.EventArgs)
        SelectAll()
    End Sub

    Private Sub OnDeselectAll(ByVal sender As System.Object, ByVal e As System.EventArgs)
        ClearSelection()
    End Sub

    Private Sub OnPrint(ByVal sender As System.Object, ByVal e As System.EventArgs)
        _parent.Print()
    End Sub

End Class

